@@ -2,8 +2,7 @@ import re
 
 from typing import Dict, Any, Union, List, AnyStr, Optional
 
-from .from_parser import FromParser
-from .json_parser import JsonParser
+from .from_parser import DynamoDBFromParser, S3FromParser, FromParser
 from .select_parser import SelectParser
 from .where_parser import DynamoDBWhereParser, S3WhereParser, WhereParser
 from .utils import is_dict, QueryMetadata
@@ -16,12 +15,14 @@ class Parser:
         self,
         source_data: Dict[str, str],
         table_prefix: Optional[str],
-        where_parser,
+        from_parser: FromParser,
+        where_parser: WhereParser,
     ):
         # Source data is in the format: {source: json}
         # Where 'json' is one or more json documents separated by a newline
         self.documents = source_data
         self.table_prefix = table_prefix
+        self.from_parser = from_parser
         self.where_parser = where_parser
 
     def parse(self, query: str, parameters=None) -> List[Dict[str, Any]]:
@@ -30,9 +31,9 @@ class Parser:
         # First clause is whatever comes in front of SELECT - which should be nothing
         _ = clauses[0]
         # FROM
-        from_clauses = FromParser().parse(clauses[2])
-        source_data = self.documents[list(from_clauses.values())[0].lower()]
-        source_data = JsonParser().parse(source_data)
+        from_parser = self.from_parser()
+        from_clauses = from_parser.parse(clauses[2])
+        source_data = from_parser.get_source_data(self.documents)
         if is_dict(source_data):
             source_data = [source_data]  # type: ignore
 
@@ -43,7 +44,11 @@ class Parser:
 
         # SELECT
         select_clause = clauses[1]
-        return SelectParser(self.table_prefix).parse(
+        table_prefix = self.table_prefix
+        for alias_key, alias_value in from_clauses.items():
+            if table_prefix == alias_value:
+                table_prefix = alias_key
+        return SelectParser(table_prefix).parse(
             select_clause, from_clauses, source_data
         )
 
@@ -51,14 +56,20 @@ class Parser:
 class S3SelectParser(Parser):
     def __init__(self, source_data: Dict[str, str]):
         super().__init__(
-            source_data, table_prefix="s3object", where_parser=S3WhereParser
+            source_data,
+            table_prefix="s3object",
+            from_parser=S3FromParser,
+            where_parser=S3WhereParser,
         )
 
 
 class DynamoDBStatementParser(Parser):
     def __init__(self, source_data: Dict[str, str]):
         super().__init__(
-            source_data, table_prefix=None, where_parser=DynamoDBWhereParser
+            source_data,
+            table_prefix=None,
+            from_parser=DynamoDBFromParser,
+            where_parser=DynamoDBWhereParser,
         )
 
     @classmethod
